package com.ym.mec.auth.web.controller;

import com.ym.mec.auth.api.entity.SysRole;
import com.ym.mec.auth.config.constant.SecurityConstants;
import com.ym.mec.auth.service.SysMenuService;
import com.ym.mec.auth.service.SysRoleMenuService;
import com.ym.mec.auth.service.SysRoleService;
import com.ym.mec.common.controller.BaseController;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import java.util.Date;

@RestController()
@RequestMapping("role")
@Api(description = "角色服务")
public class RoleController extends BaseController {

    @Autowired
    private SysRoleService sysRoleService;
    @Autowired
    private SysRoleMenuService sysRoleMenuService;
    @Autowired
    private SysMenuService sysMenuService;

    @ApiOperation("根据权限id查询角色")
    @GetMapping("/{id}")
    public Object getRole(@ApiParam(value = "权限编号", required = true) @PathVariable("id") Integer id) {
        return succeed(sysRoleService.get(id));
    }

    @ApiOperation("删除角色")
    @DeleteMapping("/del/{id}")
    public Object delRole(@ApiParam(value = "权限编号", required = true) @PathVariable("id") Integer id) {
        return succeed(sysRoleService.delete(id));
    }

    @ApiOperation("修改角色")
    @PutMapping("/update")
    public Object updateRole(SysRole sysRole) {
        sysRole.setUpdateTime(new Date());
        sysRoleService.update(sysRole);
        return succeed();
    }

    @ApiOperation("新增角色")
    @PostMapping("/add")
    public Object addRole(SysRole sysRole) {
        Date date = new Date();
        sysRole.setUpdateTime(date);
        sysRole.setCreateTime(date);
        sysRoleService.insert(sysRole);
        return succeed();
    }

    @ApiOperation("角色新增菜单权限(批量)")
    @ApiImplicitParams({ @ApiImplicitParam(name = "roleId", value = "角色编号", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "menuIds", value = "菜单id,逗号分隔", required = true, dataType = "String") })
    @PostMapping("/addRoleMenu")
    public Object addRoleMenu(Integer roleId,String menuIds) {
        if(roleId == null || StringUtils.isEmpty(menuIds)){
            return failed(SecurityConstants.PARAM_VERIFY_EXCEPTION);
        }
        sysRoleMenuService.batchInsert(roleId,menuIds);
        return succeed();
    }

    @ApiOperation("角色删除菜单权限(批量)")
    @ApiImplicitParams({ @ApiImplicitParam(name = "roleId", value = "角色编号", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "menuIds", value = "菜单id,逗号分隔", required = true, dataType = "String") })
    @DeleteMapping("/delRoleMenu")
    public Object delRoleMenu(Integer roleId,String menuIds) {
        if(roleId == null || StringUtils.isEmpty(menuIds)){
            return failed(SecurityConstants.PARAM_VERIFY_EXCEPTION);
        }
        sysRoleMenuService.batchdel(roleId,menuIds);
        return succeed();
    }

    @ApiOperation("根据角色编号查询拥有的菜单列表")
    @ApiImplicitParams({ @ApiImplicitParam(name = "roleId", value = "角色编号", required = true, dataType = "Integer")})
    @GetMapping("/getMenus")
    public Object getMenus(Integer roleId) {
        if(roleId == null){
            return failed(SecurityConstants.PARAM_VERIFY_EXCEPTION);
        }
        return succeed(sysMenuService.getMenusByRoleId(roleId));
    }
}
