package com.ym.mec.common.exception;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;

@ControllerAdvice
public class BasicControllerAdvice extends BaseController {

	private final static Logger logger = LoggerFactory.getLogger(BasicControllerAdvice.class);

	/**
	 * 处理一般异常
	 * 
	 * @param ex
	 * @param request
	 * @return
	 */
	@ExceptionHandler(Exception.class)
	@ResponseBody
	public HttpResponseResult handleException(Exception ex, HttpServletRequest request) {
		Throwable e = ExceptionUtils.getRootCause(ex);
		if (e == null) {
			e = ex;
		}

		if (e instanceof AccessDeniedException) {
			logger.error("Access Denied", e);
			return failed(HttpStatus.FORBIDDEN.value(), "禁止访问");
		}

		logger.error("System Error", e);
		return failed(e.getMessage());
	}

}
